export function listToTree(hikeElement, prefix = "!") {
    const { children } = hikeElement;
    const root = {
        depth: 0,
        sections: [],
        children: [],
        code: [],
        images: [],
    };
    let parent = root;
    children.forEach((child) => {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k;
        if (child.type === "heading" &&
            ((_a = child.children[0]) === null || _a === void 0 ? void 0 : _a.type) === "text" &&
            ((_c = (_b = child.children[0]) === null || _b === void 0 ? void 0 : _b.value) === null || _c === void 0 ? void 0 : _c.trim().startsWith(prefix))) {
            while (parent.depth >= child.depth && parent.parent) {
                parent = parent.parent;
            }
            const { name, query } = parseHeading(child, prefix);
            const section = {
                parent,
                depth: child.depth,
                name,
                query,
                sections: [],
                children: [],
                code: [],
                images: [],
            };
            parent.sections.push(section);
            parent.children.push(placeholder(name));
            parent = section;
        }
        else if (child.type === "heading" &&
            ((_d = child.children[0]) === null || _d === void 0 ? void 0 : _d.type) === "text" &&
            ((_f = (_e = child.children[0]) === null || _e === void 0 ? void 0 : _e.value) === null || _f === void 0 ? void 0 : _f.trim()) === "/") {
            parent = parent.parent || parent;
        }
        else if (child.type === "code" && !((_g = child.meta) === null || _g === void 0 ? void 0 : _g.includes("!ch-exclude"))) {
            parent.children.push(placeholder("code"));
            parent.code.push(child);
        }
        else if (
        // ![!name query](image.png)
        child.type === "paragraph" &&
            child.children.length === 1 &&
            child.children[0].type === "image" &&
            ((_h = child.children[0].alt) === null || _h === void 0 ? void 0 : _h.startsWith("!"))) {
            const img = child.children[0];
            const name = ((_j = img.alt) === null || _j === void 0 ? void 0 : _j.trim().slice(1).split(/\s+/)[0]) || "";
            const alt = (_k = img.alt) === null || _k === void 0 ? void 0 : _k.trim().slice(name.length + 1).trim();
            img.url;
            const key = name || "image";
            parent.images.push({
                key,
                value: {
                    ...img,
                    alt,
                },
            });
            parent.children.push(placeholder(key));
        }
        else {
            parent.children.push(child);
        }
    });
    return root;
}
function placeholder(name) {
    return {
        type: "mdxJsxFlowElement",
        name: "slot",
        attributes: [
            {
                type: "mdxJsxAttribute",
                name: "name",
                value: name,
            },
        ],
        children: [],
    };
}
export function isHikeHeading(child, prefix) {
    var _a, _b, _c;
    return (child.type === "heading" &&
        ((_a = child.children[0]) === null || _a === void 0 ? void 0 : _a.type) === "text" &&
        ((_c = (_b = child.children[0]) === null || _b === void 0 ? void 0 : _b.value) === null || _c === void 0 ? void 0 : _c.trim().startsWith(prefix)));
}
function parseHeading(heading, prefix) {
    var _a, _b, _c;
    if (((_a = heading.children[0]) === null || _a === void 0 ? void 0 : _a.type) != "text") {
        throw new Error("Heading must have text");
    }
    const content = (_c = (_b = heading.children[0]) === null || _b === void 0 ? void 0 : _b.value) === null || _c === void 0 ? void 0 : _c.trim().slice(prefix.length);
    const name = content === null || content === void 0 ? void 0 : content.split(/\s+/)[0];
    const query = content === null || content === void 0 ? void 0 : content.slice(name.length).trim();
    return {
        name: name || "steps",
        query,
    };
}
