import { getObjectAttribute } from "./estree.js";
export function treeToAttribute(root) {
    const children = [];
    const serializableTree = serializeTree(root);
    const queue = [root];
    while (queue.length > 0) {
        const node = queue.shift();
        children.push(sectionContainer(node));
        queue.push(...node.sections);
    }
    return {
        children,
        attributes: [
            {
                type: "mdxJsxAttribute",
                name: "__hike",
                value: getObjectAttribute(serializableTree),
            },
        ],
    };
}
function serializeTree(node) {
    const { path, query, code, sections, images } = node;
    const result = {
        query,
        children: path.join("."),
    };
    if (code === null || code === void 0 ? void 0 : code.length) {
        result.code = code;
    }
    if (images === null || images === void 0 ? void 0 : images.length) {
        images.forEach((image) => {
            const { alt, title, url } = image.value;
            result[image.key] = { alt, title, url };
        });
    }
    sections.forEach((section) => {
        result[section.name] = result[section.name] || [];
        result[section.name].push(serializeTree(section));
    });
    return result;
}
function sectionContainer(section) {
    const path = section.path.join(".");
    return {
        type: "mdxJsxFlowElement",
        name: "slot",
        attributes: [
            {
                type: "mdxJsxAttribute",
                name: "path",
                value: path,
            },
        ],
        children: section.children,
    };
}
