import { BlockContent, Code, DefinitionContent } from "mdast";
import { MdxJsxFlowElement } from "mdast-util-mdx-jsx";
import { CodeHikeConfig } from "./config.js";
export type JSXChild = BlockContent | DefinitionContent;
interface HikeNodeMap {
    section: HikeSection;
    code: HikeCode;
    image: HikeImage;
    quote: HikeQuote;
    content: HikeContent;
}
type HikeNode = HikeNodeMap[keyof HikeNodeMap];
interface HikeNodeBase {
    name: string;
    multi: boolean;
    index?: number;
}
export interface HikeSection extends HikeNodeBase {
    _data: {
        header: string;
    };
    type: "section";
    title: string;
    depth: number;
    parent: HikeSection | null;
    children: HikeNode[];
}
interface HikeCode extends HikeNodeBase {
    type: "code";
    value: string;
    lang?: string | null | undefined;
    meta?: string | null | undefined;
}
interface HikeImage extends HikeNodeBase {
    type: "image";
    alt: string;
    title: string;
    url: string;
}
interface HikeQuote extends HikeNodeBase {
    type: "quote";
    value: string;
}
export interface HikeContent {
    type: "content";
    value: JSXChild;
}
export declare function listToSection(hikeElement: MdxJsxFlowElement, config: CodeHikeConfig): Promise<HikeSection>;
export declare function isHikeElement(child: any): any;
export declare function parseCode(code: Code, config: CodeHikeConfig): Promise<{
    value: string;
    lang: string;
    meta: string;
}>;
export {};
//# sourceMappingURL=1.1.remark-list-to-section.d.ts.map