import { SKIP, visit } from "estree-util-visit";
export function transformHikeProps(root) {
    const { functionDeclaration, returnStatement } = findFunctionNodes(root);
    const { jsxOn, rootHike } = detectUsage(returnStatement);
    forEachHikeElementMoveChildrenToHikeProp(root, jsxOn);
    if (rootHike) {
        extractBlocksToConstAndConditionallyReturnThem(rootHike, functionDeclaration, returnStatement, jsxOn);
    }
    return root;
}
// find the _createMdxContent function and its return statement
function findFunctionNodes(root) {
    const { body } = root;
    const functionDeclaration = body.find(({ type, id }) => type === "FunctionDeclaration" && (id === null || id === void 0 ? void 0 : id.name) === "_createMdxContent");
    const [returnStatement] = find(functionDeclaration, (node) => node.type === "ReturnStatement");
    return { functionDeclaration, returnStatement };
}
// detect if compiler output is jsx and also if there is a root hike
function detectUsage(returnStatement) {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j;
    const returningElement = returnStatement.argument;
    const jsxOn = returningElement.type.startsWith("JSX");
    let rootHike = null;
    // if there are hike elements not wrapped in a JSX element
    if (jsxOn &&
        (returningElement === null || returningElement === void 0 ? void 0 : returningElement.type) === "JSXElement" &&
        ((_b = (_a = returningElement === null || returningElement === void 0 ? void 0 : returningElement.openingElement) === null || _a === void 0 ? void 0 : _a.attributes) === null || _b === void 0 ? void 0 : _b.some((a) => { var _a; return ((_a = a === null || a === void 0 ? void 0 : a.name) === null || _a === void 0 ? void 0 : _a.name) === "__hike"; })) &&
        ((_e = (_d = (_c = returningElement === null || returningElement === void 0 ? void 0 : returningElement.openingElement) === null || _c === void 0 ? void 0 : _c.name) === null || _d === void 0 ? void 0 : _d.property) === null || _e === void 0 ? void 0 : _e.name) == "slot") {
        rootHike = returningElement;
    }
    if (!jsxOn &&
        (returningElement === null || returningElement === void 0 ? void 0 : returningElement.type) === "CallExpression" &&
        ((_g = (_f = returningElement === null || returningElement === void 0 ? void 0 : returningElement.arguments[1]) === null || _f === void 0 ? void 0 : _f.properties) === null || _g === void 0 ? void 0 : _g.some((a) => { var _a; return ((_a = a === null || a === void 0 ? void 0 : a.key) === null || _a === void 0 ? void 0 : _a.name) === "__hike"; })) &&
        ((_j = (_h = returningElement === null || returningElement === void 0 ? void 0 : returningElement.arguments[0]) === null || _h === void 0 ? void 0 : _h.property) === null || _j === void 0 ? void 0 : _j.name) == "slot") {
        rootHike = returningElement;
    }
    return { jsxOn, rootHike };
}
function extractBlocksToConstAndConditionallyReturnThem(rootHike, functionDeclaration, returnStatement, jsxOn) {
    const blocksConst = {
        type: "VariableDeclaration",
        kind: "const",
        declarations: [
            {
                type: "VariableDeclarator",
                id: { type: "Identifier", name: "_blocks" },
                init: jsxOn
                    ? rootHike.openingElement.attributes[0].argument
                    : rootHike.arguments[1],
            },
        ],
    };
    // add blocks before return
    functionDeclaration.body.body.splice(functionDeclaration.body.body.indexOf(returnStatement), 0, blocksConst);
    const ifStatement = {
        type: "IfStatement",
        // test if props._returnBlocks is truthy
        test: {
            type: "MemberExpression",
            object: { type: "Identifier", name: "props" },
            property: { type: "Identifier", name: "_returnBlocks" },
        },
        consequent: {
            type: "BlockStatement",
            body: [
                {
                    type: "ReturnStatement",
                    // return _blocks
                    argument: {
                        type: "Identifier",
                        name: "_blocks",
                    },
                },
            ],
        },
    };
    // add if before return
    functionDeclaration.body.body.splice(functionDeclaration.body.body.indexOf(returnStatement), 0, ifStatement);
    // change return to _blocks.children
    returnStatement.argument = {
        type: "MemberExpression",
        object: { type: "Identifier", name: "_blocks" },
        property: { type: "Identifier", name: "children" },
    };
}
function forEachHikeElementMoveChildrenToHikeProp(root, jsxOn) {
    visit(root, (node) => {
        if (isElementWithHikeAttribute(node, jsxOn)) {
            if (jsxOn) {
                moveChildrenToHikePropJSX(node, jsxOn);
            }
            else {
                moveChildrenToHikeProp(node);
            }
        }
    });
}
function isElementWithHikeAttribute(node, jsxOn) {
    var _a, _b, _c, _d;
    return jsxOn
        ? (node === null || node === void 0 ? void 0 : node.type) === "JSXElement" &&
            ((_b = (_a = node === null || node === void 0 ? void 0 : node.openingElement) === null || _a === void 0 ? void 0 : _a.attributes) === null || _b === void 0 ? void 0 : _b.some((a) => { var _a; return ((_a = a === null || a === void 0 ? void 0 : a.name) === null || _a === void 0 ? void 0 : _a.name) === "__hike"; }))
        : (node === null || node === void 0 ? void 0 : node.type) === "CallExpression" &&
            ((_d = (_c = node === null || node === void 0 ? void 0 : node.arguments[1]) === null || _c === void 0 ? void 0 : _c.properties) === null || _d === void 0 ? void 0 : _d.some((a) => { var _a; return ((_a = a === null || a === void 0 ? void 0 : a.key) === null || _a === void 0 ? void 0 : _a.name) === "__hike"; }));
}
function moveChildrenToHikePropJSX(node, jsxOn) {
    // dictionary of children by path
    const childrenByPath = {};
    node.children.forEach((slot) => {
        const path = slot.openingElement.attributes.find((a) => a.name.name === "path").value.value;
        childrenByPath[path] = childrenByPath[path] || [];
        childrenByPath[path].push(slot.children);
    });
    // replace all the `children` props inside `__hike` with the actual children
    const { attributes } = node.openingElement;
    const hikeAttributeIndex = attributes.findIndex((a) => a.name.name === "__hike");
    const hikeAttribute = attributes[hikeAttributeIndex];
    visit(hikeAttribute, function (node) {
        var _a;
        if ((node === null || node === void 0 ? void 0 : node.type) === "Property" && ((_a = node === null || node === void 0 ? void 0 : node.key) === null || _a === void 0 ? void 0 : _a.value) === "children") {
            const path = node.value.value;
            const elements = childrenByPath[path].shift();
            node.value = elements[0];
            return SKIP;
        }
    });
    // remove children from the hike element
    node.children = [];
    // remove the `__hike` prop from the attributes
    attributes.splice(hikeAttributeIndex, 1);
    // spread the `__hike` prop to the beginning of the attributes
    attributes.unshift({
        type: "JSXSpreadAttribute",
        argument: hikeAttribute.value.expression,
    });
}
function moveChildrenToHikeProp(node) {
    // dictionary of children by path
    const childrenByPath = {};
    const childrenExpression = node.arguments[1].properties.find((a) => a.key.name === "children").value;
    const children = childrenExpression.type === "ArrayExpression"
        ? childrenExpression.elements
        : [childrenExpression];
    children.forEach((callExpression) => {
        var _a;
        const props = (_a = callExpression.arguments[1]) === null || _a === void 0 ? void 0 : _a.properties;
        const path = props.find((p) => p.key.name === "path").value.value;
        let slotChildren = props.find((p) => p.key.name === "children").value;
        childrenByPath[path] = childrenByPath[path] || [];
        childrenByPath[path].push(slotChildren);
    });
    // replace all the `children` props inside `__hike` with the actual children
    const { properties } = node.arguments[1];
    const hikePropertyIndex = properties.findIndex((a) => a.key.name === "__hike");
    const hikeProperty = properties[hikePropertyIndex];
    visit(hikeProperty, function (node) {
        var _a;
        if ((node === null || node === void 0 ? void 0 : node.type) === "Property" && ((_a = node === null || node === void 0 ? void 0 : node.key) === null || _a === void 0 ? void 0 : _a.value) === "children") {
            const path = node.value.value;
            const elements = childrenByPath[path].shift();
            node.value = elements;
            return SKIP;
        }
    });
    // remove the `__hike` prop from the attributes
    properties.splice(hikePropertyIndex, 1);
    // spread the `__hike` prop to the beginning of the attributes
    properties.unshift(...hikeProperty.value.properties);
    // remove children from the hike element
    node.arguments[1].properties = properties.filter((p) => p.key.name !== "children");
    // jsxs calls can only have arrays as children, so we turn any jsxs without array into jsx call
    visit(node, function (node) {
        if (node.type === "CallExpression" && node.callee.name === "_jsxs") {
            const childrenProp = node.arguments[1].properties.find((p) => p.key.value === "children");
            if (childrenProp && childrenProp.value.type !== "ArrayExpression") {
                node.callee.name = "_jsx";
            }
        }
    });
}
function find(node, predicate) {
    let result = [];
    visit(node, (node, key, index, ancestors) => {
        if (predicate(node)) {
            result = [node, ...ancestors.slice().reverse()];
            return SKIP;
        }
    });
    return result;
}
