import { jsx as _jsx } from "react/jsx-runtime";
import React from "react";
const ObserverContext = React.createContext(undefined);
export function Scroller({ onIndexChange, triggerPosition = "50%", children, }) {
    const [observer, setObserver] = React.useState();
    const vh = useWindowHeight();
    React.useLayoutEffect(() => {
        const windowHeight = vh || 0;
        const handleIntersect = (entries) => {
            entries.forEach((entry) => {
                if (entry.intersectionRatio > 0) {
                    // get entry.target index
                    const index = entry.target.getAttribute("data-index");
                    onIndexChange(+index);
                }
            });
        };
        const observer = newIntersectionObserver(handleIntersect, defaultRootMargin(windowHeight, triggerPosition));
        setObserver(observer);
        return () => observer.disconnect();
    }, [vh]);
    return (_jsx(ObserverContext.Provider, { value: observer, children: children }));
}
function newIntersectionObserver(handleIntersect, rootMargin) {
    return new IntersectionObserver(handleIntersect, {
        rootMargin,
        threshold: 0.000001,
        root: null,
    });
}
export function ObservedDiv({ index, ...props }) {
    const ref = React.useRef(null);
    const observer = React.useContext(ObserverContext);
    React.useLayoutEffect(() => {
        if (observer && ref.current) {
            observer.observe(ref.current);
        }
        return () => {
            observer && ref.current && observer.unobserve(ref.current);
        };
    }, [observer]);
    return _jsx("div", { ref: ref, ...props, "data-index": index });
}
function useWindowHeight() {
    const isClient = typeof window === "object";
    function getHeight() {
        return isClient ? document.documentElement.clientHeight : undefined;
    }
    const [windowHeight, setWindowHeight] = React.useState(getHeight);
    React.useEffect(() => {
        function handleResize() {
            setWindowHeight(getHeight());
        }
        window.addEventListener("resize", handleResize);
        return () => window.removeEventListener("resize", handleResize);
    }, []);
    React.useLayoutEffect(() => {
        // FIX when a horizontal scrollbar is added after the first layout
        setWindowHeight(getHeight());
    }, []);
    return windowHeight;
}
function defaultRootMargin(vh, triggerPosition = "50%") {
    let y = vh * 0.5;
    if (triggerPosition.endsWith("%")) {
        const percent = parseFloat(triggerPosition.replace("%", ""));
        y = vh * (percent / 100);
    }
    else if (triggerPosition.endsWith("px")) {
        y = parseFloat(triggerPosition.replace("px", ""));
    }
    if (y < 0) {
        y = vh + y;
    }
    return `-${y - 2}px 0px -${vh - y - 2}px`;
}
